@@ -4,8 +4,7 @@
 
 package Exporter::Proxy;
 
-use v5.12;
-use strict;
+use v5.20;
 
 use Carp;
 
@@ -16,7 +15,7 @@ use Symbol      qw( qualify_to_ref  );
 # package variables
 ########################################################################
 
-our $VERSION    = '1.4';
+our $VERSION    = '1.7';
 $VERSION = eval $VERSION;
 
 my $disp_list   = 'DISPATCH_OK';
@@ -53,8 +52,10 @@ sub import
 
     # maybe carp about extraneous arguments here?
 
-    my $disp    = delete $argz{ dispatch    } || '';
-    my $preproc = delete $argz{ prefilter   } || '';
+    my $disp        = delete $argz{ dispatch    } || '';
+    my $preproc     = delete $argz{ prefilter   } || '';
+    my $postproc    = delete $argz{ postfilter  } || '';
+    my $inst_import = delete $argz{ import      } || 1;
 
     # if a dispatcher is being used then it must
     # be exported. in most cases this will be the
@@ -112,32 +113,12 @@ sub import
         }
     }
 
-    my $pre_handler
-    = do
-    {
-        if( $preproc )
-        {
-            sub
-            {
-                my $handler = $source->can( $preproc )
-                or die "$source cannot '$preproc'\n";
-
-                &$handler
-            }
-        }
-        else
-        {
-            state $stub = sub{}
-        }
-    };
-
     @exportz
     or carp "Oddity: nothing requested for export!";
 
     my $exports = qualify_to_ref 'exports', $source;
-    my $import  = qualify_to_ref 'import',  $source;
 
-    undef &{ *$_ } for ( $exports, $import );
+    undef &$exports;
 
     *$exports
     = sub
@@ -150,58 +131,98 @@ sub import
         : [ @exportz ]
     };
 
-    *$import
-    = sub
+    $inst_import
+    and
+    do
     {
-        # discard the package as first argument:
-        # $pkg->import
+        my $import  = qualify_to_ref 'import',  $source;
 
-        shift;
+        undef &$import;
 
-        my $caller  = caller;
-        
-        # allow the caller to pre-process the arguments.
-        # notice this happens *before* ":noexport" is 
-        # processed.
-
-        &$pre_handler;
-
-        # empty list => use @exportz.
-        # :noexport  => use empty list.
-
-        if( first { ':noexport' eq $_ } @_ )
-        {
-            @_  = ();
-        }
-        elsif( @_ )
-        {
-            # nothing more for the moment.
-        }
-        else
-        {
-            @_  = @exportz;
-        }
-
-        # resolve these at runtime to account for
-        # possible autoloading, etc.
-
-        for my $arg ( @_ )
-        {
-            index $arg, ':'
-            or next;
-
-            if( first { $arg eq $_ }  @exportz )
+        my $find_pre    
+        = $preproc
+        ?   sub 
             {
-                my $source  = qualify_to_ref $arg, $source;
-                my $install = qualify_to_ref $arg, $caller;
+                $source->can( $preproc )
+                or die "Unusable $source: cannot '$preproc'";
+            }
+        : $stub
+        ;
 
-                *$install   = *$source;
+        my $find_post    
+        = $postproc
+        ?   sub 
+            {
+                $source->can( $postproc )
+                or die "Unusable $source: cannot '$postproc'";
+            }
+        : $stub
+        ;
+
+        *$import
+        = sub
+        {
+            # these are delayed since the "use E::P" is 
+            # usually dealt with before the subs are defined
+            # in the caller.
+
+            state $pre_handler  = $find_pre->();
+            state $post_handler = $find_post->();
+
+            # discard the package as first argument:
+            # $pkg->import
+
+            shift;
+
+            my $caller  = caller;
+            
+            # allow the caller to pre-process the arguments.
+            # notice this happens *before* ":noexport" is 
+            # processed.
+
+            &$pre_handler
+            if $pre_handler;
+
+            # empty list => use @exportz.
+            # :noexport  => use empty list.
+
+            if( first { ':noexport' eq $_ } @_ )
+            {
+                @_  = ();
+            }
+            elsif( @_ )
+            {
+                # nothing more for the moment.
             }
             else
             {
-                die "Bogus $source: '$arg' not exported";
+                @_  = @exportz;
             }
-        }
+
+            # resolve these at runtime to account for
+            # possible autoloading, etc.
+
+            for my $arg ( @_ )
+            {
+                index $arg, ':'
+                or next;
+
+                if( first { $arg eq $_ }  @exportz )
+                {
+                    my $source  = qualify_to_ref $arg, $source;
+                    my $install = qualify_to_ref $arg, $caller;
+
+                    *$install   = *$source;
+                }
+                else
+                {
+                    die "Bogus $source: '$arg' not exported";
+                }
+            }
+
+            goto &$post_handler
+            if $post_handler;
+        };
     };
 
     return
@@ -250,7 +271,7 @@ Exporter::Proxy - Simplified symbol export & proxy dispatch.
     # subroutine, Bar as an array, or foo may 
     # be overloaded with &foo, %foo, @foo, $foo.
 
-    $value ~~ $Bar[0]
+    $value eq $Bar[0]
     or croak "Invalid '$value'";
 
     delete $foo{ somekey }
@@ -258,9 +279,8 @@ Exporter::Proxy - Simplified symbol export & proxy dispatch.
 
     my $bletch  = $foo || 'oops, no $foo';
 
-    # if the caller does not want to export
-    # anything from the module when it is used,
-    # ":noexport" does just that.
+    # if the caller does not want to import
+    # anything from the module by default:
 
     use My::Module qw( :noexport );
 
@@ -299,6 +319,21 @@ Exporter::Proxy - Simplified symbol export & proxy dispatch.
     # modifying $test_ref does not affect
     # the exported items.
 
+    # some modules may want to use their own import, pulling
+    # in only exports and the dispatcher.
+
+    package Foo::Bar;
+
+    use Exporter::Proxy qw( import=0 );
+
+    sub import
+    {
+        # left as-is, package has to implement its own
+        # export utility for @exports.
+
+        ...
+    }
+
 =head1 DESCRIPTION
 
 This installs 'import' and 'exports' subroutines 
@@ -345,8 +380,8 @@ With no arguments the import uses the original
 exports list, pushing all of the symbols into
 the caller's space.
 
-The optional argument ':noexport' short-circuts
-export of any symbols to the caller's space.
+The optional argument ':noexport' avoids exporting
+any symbols to the caller's space.
 
 Other than ':noexport' any arguments with leading
 colons are silently ignored by import.
@@ -510,6 +545,23 @@ Aside: In later versions of perl
 
 makes it easier to get $i and the value.
 
+The postfilter is called as a final step in the import sub
+via goto &$post_handler. It cannot affect the stack used by 
+import but will have access to caller() for determining the
+calling class. THi can be usefuil for cases where some 
+information is managed by class such as default files or 
+per-package counts:
+
+    use Exporter::Proxy qw( postfilter=store_exports ... );
+
+    sub store_exports
+    {
+        # this returns the package calling import
+
+        my $caller  = caller;
+
+        $info{ $caller } = ...
+    }
 
 =back
 
@@ -645,6 +697,6 @@ Steven Lembark <lembark@wrkhors.com>
 
 =head1 LICENSE
 
-Copyright (C) 2009 Workhorse Computing.
-This code is released under the same terms as Perl 5.10.0,
+Copyright (C) 2009-2016 Workhorse Computing.
+This code is released under the same terms as Perl 5.22,
 or any later version of Perl, itself.
